# Diary

This is a text-based diary, inspired by [khal](https://github.com/pimutils/khal). Journal entries are stored in text files.

## Demo
![Diary Demo](https://codeberg.org/in0rdr/diary/raw/branch/master/img/demo.gif)

[Quickstart Tutorial](https://diary.in0rdr.ch)

## Usage
1. Set the EDITOR environment variable to your favourite text editor:
    ```
    export EDITOR=vim
    ```

2. Run the diary, with the folder for the text files as first argument:
    ```
    diary ~/.diary
    ```

   Instead of this, you can also set the environment variable `DIARY_DIR`
   to the desired directory. If both an argument and the environment
   variable are given, the argument takes precedence (see [Variable Precedence Rules](#precedence_rules)).

   The text files in this folder will be named 'yyyy-mm-dd'.

3. Use the keypad or VIM-like shortcuts to move between dates:

    ```
    e, Enter  edit current entry
    d, x      delete current entry

    t         jump to today
    f         jump to or find specific day

    j, down   go forward by 1 week
    k, up     go backward by 1 week
    h, left   go left by 1 day
    l, right  go right by 1 day
    J         go forward by 1 month
    K         go backward by 1 month

    N         go to the previous journal entry
    n         go to the next journal entry
    g         go to start of journal
    G         go to end of journal

    s, S      sync selected day/Month with CalDAV server
    i         import entries from .ics file
    E         export entries to .ics file

    q         quit the program
    ```

Keybinding Cheat Sheet:

![diary cheat sheet](https://codeberg.org/in0rdr/diary/raw/branch/master/img/diary-cheat-sheet.png)

## Install

Pre-built packages can be installed from the [Open Build Service (OBS)](https://software.opensuse.org/download.html?project=home%3Ain0rdr&package=diary)
* Install the `diary` package for a stable releases
* Install the `diary-nightly` build to use a more recent but possibly unstable version from the master branch

### Arch Linux OBS Repository Bug

The Arch Linux repository on the OBS is broken, see [#69](https://github.com/in0rdr/diary/issues/69) and [#9953](https://github.com/openSUSE/open-build-service/issues/9953).

To use the repository on Arch Linux, use `downloadcontent.opensuse.org` instead of the [suggested](https://software.opensuse.org/download.html?project=home%3Ain0rdr&package=diary) name `download.opensuse.org` in the `/etc/pacman.conf`:
```
[home_in0rdr_Arch]
Server = https://downloadcontent.opensuse.org/repositories/home:/in0rdr/Arch/$arch
```

## Build from Source

1. Define [OAuth2 application credentials](https://developers.google.com/identity/protocols/oauth2) if CalDAV sync should be effective:
    ```
    export GOOGLE_OAUTH_CLIENT_ID=""
    export GOOGLE_OAUTH_CLIENT_SECRET=""
    ```

    Alternatively, leave these two variables unset and [define the clientid/secret in the configuration file](#Google-Calendar-OAuth2) at run-time.

2. Compile (requires ncurses and libcurl):
    ```
    make
    ```
Note: for *BSD users run gmake.

3. Install the binary (optional):
    ```
    sudo make install
    ```

   By default this will copy the binary to /usr/local/bin. To use a different
   path prefix, type `sudo make PREFIX=/usr install` to use /usr/bin for example.
   You can uninstall diary with `sudo make uninstall`.

## OBS Build Documentation
[./docs/OBS.md](./docs/OBS.md)

## Test Documentation
[./docs/TESTING.md](./docs/TESTING.md)

## Configuration File

The [`diary.cfg`](./config/diary.cfg) configuration file can optionally be used to persist diary configuration. To install the sample from this repository:
```bash
mkdir -p ${XDG_CONFIG_HOME:-~/.config}/diary
cp config/diary.cfg ${XDG_CONFIG_HOME:-~/.config}/diary/
```

The file `${XDG_CONFIG_HOME:-~/.config}/diary/diary.cfg` should adhere to a basic `key = value` format. Lines can be commented with the special characters `#` or `;`. The following configuration keys are currently supported:

| Command Line Option | Config Key | Example Config Value | Default Config Value | Description |
| --- | --- | --- | --- | --- |
| `--dir`, `-d`, or first non-option argument | `dir` | ~/diary | n/a | Diary directory. Path that holds the journal text files. If unset, defaults to environment variable `$DIARY_DIR`.|
| `--editor` or `-e` | `editor` | vim | (empty) | Editor to open journal files with. If unset, defaults to environment variable `$EDITOR`. If no editor is provided, the diary is opened read-only. |
| `--fmt` or `-f` | `fmt` | %d_%b_%y | %Y-%m-%d | Date format and file name for the files inside the `dir`. For the format specifiers, see [`man strftime`](https://man7.org/linux/man-pages/man3/strftime.3.html). Be careful: If you change this, you might no longer find your existing diary entries, because the diary assumes to find the journal files under another file name. Hence, a change in FMT shows an empty diary, at first. Rename all files in the DIARY_DIR to migrate to a new FMT. |
| `--fmt-cmd` or `-F` | `fmt_cmd` | `fmt -w75 -s`, `mdcat -c` or `mdcat` | (empty) | Journal entries are formatted with this command. If not set (default), the characters are printed to the preview screen character by character (without word wrap). For example, to enable word wrap after 75 characters, use "fmt -s" as FMT_CMD. See also [Custom Formatting](#Custom-Formatting). |
| `--no-pty` or `-p` | `no_pty` | `true` | false (or 0) | No pseudo terminal (pty) for entry preview. Only effective in combination with `--fmt-cmd`. If set to true (or 1), the output of `--fmt-cmd` is displayed with Ncurses. If set to false (or 0), the result of `--fmt-cmd` (journal entry preview) is printed on a pseudo-terminal (pty) and not processed with Ncurses. The pty supports the ANSI escape sequences (e.g. colors, font types, etc.) of the native terminal. |
| `--no-mouse` or `-m` | `no_mouse` | `true` | false (or 0) | Don't listen for mouse events. With `no_mouse=false` (default), native text selection and right-click in the preview window requires holding down the `SHIFT` key, see [Mouse Events](#Mouse-Events). |
| `--range` or `-r` | `range` | 10 | 1 | Number of years to show before/after todays date |
| `--weekday` or `-w` | `weekday` | 0 | 1 | First weekday, `7` = Sunday, `1` = Monday, ..., `6` = Saturday. Use `7` (or `0`) to display week beginning at Sunday ("S-M-T-W-T-F-S"), or `1` for "M-T-W-T-F-S-S". If `glibc` is installed, the first day of the week is derived from the current locale setting (`$LANG`, see `man locale`). Without `glibc`, the first weekday defaults to 1 (Monday), unless specified otherwise with this option. |
| n/a | `google_calendar` | diary | (empty) | Displayname of Google Calendar for [CalDAV sync](#CalDAV-sync) |
| n/a | `google_clientid` | 123-123.apps.googleusercontent.com | [built-in](#Build) / (empty) | Google Calendar for [Google Calendar OAuth2](#Google-Calendar-OAuth2) clientid |
| n/a | `google_secretid` | 321 | [built-in](#Build) / (empty) |  Google Calendar for [Google Calendar OAuth2](#Google-Calendar-OAuth2) secretid |
| n/a | `google_tokenfile` | ~/.diary-token | ~/.diary-token | Displayname of Google Calendar for [Google Calendar OAuth2](#Google-Calendar-OAuth2) API token file|

## Precedence Rules
<a name="precedence_rules"></a>

The default variables, for instance, for the configuration variables `editor`, `dir` and `weekday`, are populated with values in the following order, where earlier entries are overwritten by subsequent ones if they exist:
1. No default for `DIARY_DIR`. Defaults for `range`, `weekday`, `fmt` and `editor` are provided in [diary.h](diary.h). If `EDITOR` is unset and no editor is provided in the config file or via the `-e` option, the diary works read-only. Journal files cannot be opened. If `DIARY_DIR` is not provided, the diary won't open.
2. **Configuration file** (empty default for `editor`, no default for `dir`)
3. **Environment** variables `$DIARY_DIR`, `$EDITOR` and `$LANG` for locale (`weekday`)
4. **Option arguments** `-d` / `-e` / `-w`
5. First non-option argument is interpreted as `DIARY_DIR`

### Precedence Example: Locale and First Day of Week

If glibc is installed, the first weekday defaults to the locale defined in the current shell environment (`$LANG`, see `man locale`), unless specified otherwise via the `--weekday` / `-w` command line options. For example:

```bash
# start with weekday=3(Wed), overrule any other configuration value
$ diary -w3

# start with glibc derived weekday=1, regardless of 'weekday' in config file
$ LANG=de_CH diary

# if glibc is installed, start with glibc derived base date (weekday=0)
$ LANG= diary

# disable environment variable, default to value from config file
$ unset LANG

# start with 'weekday' default from config file, if available
$ diary

# remove config file
$ rm ${XDG_CONFIG_HOME:-~/.config}/diary/diary.cfg

# start with 'weekday' default value from source code (1=Mon)
$ diary
```

## Custom Formatting

The preview of the journal entries can be processed with a custom `--fmt-cmd`, `-F` or `fmt_cmd` in the [configuration file](#Configuration-File).

The result of this formatting command is printed to a pseudo-terminal (pty) by default (behavior can be changed with the `--no-pty` flag).

The output processed on the pty (not with Ncurses, without `--no-pty` flag) includes all ANSI escape characters supported by the terminal (colors, font types, etc.).

Thus, formatters like `fmt` or any "cat like program" such as [`mdcat`](https://github.com/lunaryorn/mdcat) to process Markdown can be plugged in for `--fmt-cmd`.

## Mouse Events

Mouse events are listened for by default ([`no_mouse=false`](#Configuration-File)). This behavior can be disabled with the [`--no-mouse` flag](#Configuration-File) or in the [configuration file](#Configuration-File).

With `no_mouse=false` (default), use the `SHIFT` key for native text selection and right-click in the preview window.

## Import/Export
* The import functionality can be triggered by pressing `i`.
* The export functionality can be triggered by pressing `E`.

## CalDAV Sync
The journal files can be synced via CalDAV by pressing `s`. Pressing `S` syncs all journal entries of the active month.

Use the export/import functionality for more batch oriented processing. Making a network request and deciding the most recently modified entry (local/remote) takes quite some time for larger batches of entries.

The most recently modified entry is considered up to date and is automatically replaced on the remote side. If the remote file is more recent, the local file can be replaced with the remote content after confirmation.

No sync is performed, when both files (local/remote) have the same last modified timestamp or no files exists on the server and on disk at all (i.e., no entry for that date).

Currently, only the Google Calendar is supported as remote provider. Please open an [issue](https://codeberg.org/in0rdr/diary/issues) to implement support for additional remote calendar servers.

The calender for synchronization can be defined with the [configuration](#Configuration-File) key `google_calendar`:
```
# Google calendar name for CalDAV sync
google_calendar = example
```

This key is empty by default.

### Google Calendar OAuth2

The Google Calendar [CalDAV API is protected with OAuth2](https://developers.google.com/identity/protocols/oauth2).

The credentials and the consent screen can be redefined at [compile time](#Build) or with the following keys in the [configuration file](#Configuration-File):

```
# Google OAuth2 clientid and secretid
google_clientid = 123-123.apps.googleusercontent.com
google_secretid = 321
```

The token used to authenticate with the Google API is stored in the file specified by `google_tokenfile` (defaults to `~/.diary-token`) and renewed automatically:
```
# Google OAuth2 tokenfile
google_tokenfile = ~/.diary-token
```

The application requires two [OAuth2 scopes](https://developers.google.com/calendar/auth) for CalDAV requests:

1. `https://www.googleapis.com/auth/calendar`: read/write access to Calendars - required to discover the unique hyperlink/URI for the calendar specified by the [configuration key](#Configuration-File) `google_calendar`
2. `https://www.googleapis.com/auth/calendar.events.owned`: read/write access to Events owned by the user - allows diary to create/read/update/delete events in `google_calendar`

## License

This work uses the MIT License. See [LICENSE](./LICENSE) for details.

## Contributors

All source code is available in this repository. Contributions are always welcome!

For a list of contributors see the [AUTHORS.md](./AUTHORS.md).

## Community and Support

For question and support visit [#diary/libera](https://web.libera.chat/gamja/#diary) on IRC.