# Testing

This file holds notes for testing purposes.

## Valgrind

Use Valgrind on a build with debug symbols to discover memory issues:
```bash
mkdir -p tmp-journal
make debug
valgrind --leak-check=full ./diary tmp-journal/ 2>log.txt
```

https://developers.redhat.com/blog/2021/04/23/valgrind-memcheck-different-ways-to-lose-your-memory#generating_a_leak_summary

Don't use `--show-leak-kinds=all` with Ncurses:
https://invisible-island.net/ncurses/ncurses.faq.html#config_leaks

## Compile with Debug Symbols

To make a build with debug symbols use the `debug` target:
```bash
make debug
```

## Compile with Clang Compiler

As pointed out in [#82](https://github.com/in0rdr/diary/issues/82) the clang compiler is a lot more useful when it comes to debugging.

To compile the program with clang:
```
make CC=clang
```

## Send stderr to File

Send stderr to a file for debugging:
```bash
diary 2>log.txt
```

Or to `/dev/null` to ignore the debug messages flickering on the screen:
```bash
diary 2>/dev/null
```

## Render Documentation / Man Page

### Generate Runoff
Install [scdoc](https://git.sr.ht/~sircmpwn/scdoc).

Generate the "runoff" text with `scdoc` from the Markdown like `.scd` file:
```bash
scdoc < diary.1.scd > diary.1
```

### View Manpage in Terminal
Generate plain text from "runoff" text:
```bash
tbl diary.1 | nroff -man | less
```

### `mandoc` HTML (Preferred)
Install [`mandoc`](https://en.wikipedia.org/wiki/Mandoc) >= [1.14.6 (helps to format paragraphs with CSS)](https://mandoc.bsd.lv/NEWS):
```bash
curl -sO https://mandoc.bsd.lv/snapshots/mandoc-1.14.6.tar.gz
tar -xf mandoc-1.14.6.tar.gz
cd mandoc-1.14.6
./configure
make
```

Generate HTML page from runoff:
```bash
../../mandoc-1.14.6/mandoc -Thtml -O style=style.css diary.1 > diary.1.html
```

### `groff` HTML (Alternative)

Limitation: `groff` produces unstyled output (no CSS).

Install [`groff`](https://www.gnu.org/software/groff):
```bash
sudo apt-get install groff
```

Generate HTML file (`-t` for `tbl`):
```bash
groff -t -mandoc -Thtml diary.1 > diary.1.html
```
